==> rulee-ts-0.6.1.tgz: package/lib/esm/engine/builder/property-scalar-builder.js <==
import { SimpleValueProvider } from "../../provider/value-provider/simple-value-provider";
import { SelectValueConverter } from "../../value-converter/choices/select-value-converter";
import { ChoiceValueProvider } from "../../provider/value-provider/choices/choice-value-provider";
import { ObjectValueProvider } from "../../provider/value-provider/object-value-provider";
import { C } from "../../value-converter/common-value-converters";
import { DerivedValueProvider } from "../../provider/value-provider/derived-value-provider";
import { DerivedAsyncValueProvider } from "../../provider/value-provider/derived-async-value-provider";
import { ConstantValueProvider } from "../../provider/value-provider/constant-value-provider";
import { EmptyValueFcns } from "../../provider/value-provider/empty-value-fcn";
import { ChoiceListConverter } from "../../value-converter/choices/choice-list-converter";
import { assertThat } from "../../util/assertions/assertions";
import { SelectValueProvider } from "../../provider/value-provider/choices/select-value-provider";
import { TypeaheadValueProvider } from "../../provider/value-provider/choices/typeahead-value-provider";
import { TypeaheadValueConverter } from "../../value-converter/choices/typeahead-value-converter";
import { upgradeAsPropertyWithChoices } from "../../properties/property-scalar-with-choices";
export class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new ChoiceValueProvider(choices, emptyChoice);
                const converter = new SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choices[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived1: (id, dependency, derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.sync1(`${id}__choices__`, new ChoiceListConverter(), dependency, derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            asyncDerived1: (id, dependency, derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.async1(`${id}__choices__`, new ChoiceListConverter(), dependency, derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            withPropertySource: (id, choicesSource, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new SelectValueProvider(choicesSource, emptyChoice);
                const converter = new SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], undefined, undefined, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choicesSource.getNonNullValue()[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, options) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async1(`${id}__choices__`, new ChoiceListConverter(), inputSourceProperty, {
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (options.minimumTextLength && text.length < options.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return options.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty);
            },
            withPropertySource: (id, inputSource, choicesSource) => {
                const provider = new TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], undefined, undefined, [inputSource, choicesSource]);
                return upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync1: (id, valueConverter, dependency, derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                const dependencies = [dependency];
                const invFcn = this.invFcn(derivations.inverse && ((deps, val) => derivations.inverse(deps[0], val)));
                const provider = new DerivedValueProvider(dependencies, (deps) => derivations.derive(deps[0]), invFcn);
                return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies);
            },
            sync2: (id, valueConverter, dependency1, dependency2, derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                const dependencies = [dependency1, dependency2];
                const invFcn = this.invFcn(derivations.inverse && ((deps, val) => derivations.inverse(deps[0], deps[1], val)));
                const provider = new DerivedValueProvider(dependencies, (deps) => derivations.derive(deps[0], deps[1]), invFcn);
                return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies);
            },
            async0: (id, valueConverter, derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                const dependencies = [];
                const invFcn = this.asycInvFcn(derivations.inverseAsync && ((deps, val) => derivations.inverseAsync(val)));
                const provider = new DerivedAsyncValueProvider(dependencies, (deps) => derivations.deriveAsync(), invFcn);
                return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies, undefined, derivations.backpressureConfig);
            },
            async1: (id, valueConverter, dependency, derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                const dependencies = [dependency];
                const invFcn = this.asycInvFcn(derivations.inverseAsync && ((deps, val) => derivations.inverseAsync(deps[0], val)));
                const provider = new DerivedAsyncValueProvider(dependencies, (deps) => derivations.deriveAsync(deps[0]), invFcn);
                return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies, undefined, derivations.backpressureConfig);
            },
            async2: (id, valueConverter, dependency1, dependency2, derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                const dependencies = [dependency1, dependency2];
                const invFcn = this.asycInvFcn(derivations.inverseAsync && ((deps, val) => derivations.inverseAsync(deps[0], deps[1], val)));
                const provider = new DerivedAsyncValueProvider(dependencies, (deps) => derivations.deriveAsync(deps[0], deps[1]), invFcn);
                return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies, undefined, derivations.backpressureConfig);
            }
        };
        assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        return template;
    }
    simpleProperty(id, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new SimpleValueProvider(), emptyValueFcn, valueConverter);
    }
    stringProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.string.identity, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : '');
    }
    numberProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), (options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? EmptyValueFcns.defaultEmptyValueFcn : EmptyValueFcns.numberEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.number.default, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : ((options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? 0 : null));
    }
    booleanProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.booleanEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.boolean.default, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    dateProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.date.iso, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    objectProperty(id, obj, get, set, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    objectPathProperty(id, obj, path, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        const get = (obj) => null;
        const set = (obj, val) => {
        };
        return this.propertyScalar(id, new ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    enviromentProperty(id, valueConverter, value, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        let provider;
        if (value instanceof Function) {
            provider = new DerivedValueProvider([], (deps) => value());
        }
        else {
            provider = new ConstantValueProvider(value);
        }
        return this.propertyScalar(id, provider, emptyValueFcn, valueConverter);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
    invFcn(fcn) {
        if (fcn) {
            return fcn;
        }
    }
    asycInvFcn(fcn) {
        if (fcn) {
            return fcn;
        }
    }
}

==> rulee-ts-0.7.0.tgz: package/lib/esm/engine/builder/property-scalar-builder.js <==
import { SimpleValueProvider } from "../../provider/value-provider/simple-value-provider";
import { SelectValueConverter } from "../../value-converter/choices/select-value-converter";
import { ChoiceValueProvider } from "../../provider/value-provider/choices/choice-value-provider";
import { ObjectValueProvider } from "../../provider/value-provider/object-value-provider";
import { C } from "../../value-converter/common-value-converters";
import { DerivedValueProvider } from "../../provider/value-provider/derived-value-provider";
import { DerivedAsyncValueProvider } from "../../provider/value-provider/derived-async-value-provider";
import { ConstantValueProvider } from "../../provider/value-provider/constant-value-provider";
import { EmptyValueFcns } from "../../provider/value-provider/empty-value-fcn";
import { ChoiceListConverter } from "../../value-converter/choices/choice-list-converter";
import { assertThat } from "../../util/assertions/assertions";
import { SelectValueProvider } from "../../provider/value-provider/choices/select-value-provider";
import { TypeaheadValueProvider } from "../../provider/value-provider/choices/typeahead-value-provider";
import { TypeaheadValueConverter } from "../../value-converter/choices/typeahead-value-converter";
import { upgradeAsPropertyWithChoices } from "../../properties/property-scalar-with-choices";
export class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new ChoiceValueProvider(choices, emptyChoice);
                const converter = new SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choices[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.sync(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            asyncDerived: (id, ...dependencies) => (derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), ...dependencies)(derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            withPropertySource: (id, choicesSource, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new SelectValueProvider(choicesSource, emptyChoice);
                const converter = new SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], undefined, undefined, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choicesSource.getNonNullValue()[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, options) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new ChoiceListConverter(), inputSourceProperty)({
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (options.minimumTextLength && text.length < options.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return options.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty);
            },
            withPropertySource: (id, inputSource, choicesSource) => {
                const provider = new TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], undefined, undefined, [inputSource, choicesSource]);
                return upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, valueConverter, ...dependencies) => {
                return (derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                    const provider = new DerivedValueProvider(dependencies, (deps) => derivations.derive(...deps), derivations === null || derivations === void 0 ? void 0 : derivations.inverse);
                    return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies);
                };
            },
            async: (id, valueConverter, ...dependencies) => {
                return (derivations, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) => {
                    const provider = new DerivedAsyncValueProvider(dependencies, (deps) => derivations.deriveAsync(...deps), derivations === null || derivations === void 0 ? void 0 : derivations.inverseAsync);
                    return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies, undefined, derivations.backpressureConfig);
                };
            }
        };
        assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        return template;
    }
    simpleProperty(id, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new SimpleValueProvider(), emptyValueFcn, valueConverter);
    }
    stringProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.string.identity, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : '');
    }
    numberProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), (options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? EmptyValueFcns.defaultEmptyValueFcn : EmptyValueFcns.numberEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.number.default, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : ((options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? 0 : null));
    }
    booleanProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.booleanEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.boolean.default, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    dateProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new SimpleValueProvider(), EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : C.date.iso, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    objectProperty(id, obj, get, set, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    objectPathProperty(id, obj, path, valueConverter, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        const get = (obj) => null;
        const set = (obj, val) => {
        };
        return this.propertyScalar(id, new ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    enviromentProperty(id, valueConverter, value, emptyValueFcn = EmptyValueFcns.defaultEmptyValueFcn) {
        let provider;
        if (value instanceof Function) {
            provider = new DerivedValueProvider([], (deps) => value());
        }
        else {
            provider = new ConstantValueProvider(value);
        }
        return this.propertyScalar(id, provider, emptyValueFcn, valueConverter);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
}
